SystemOrganization addCategory: #'BinaryModules-Tests'!
SystemOrganization addCategory: #BinaryModules!
SystemOrganization addCategory: #'BinaryModules-Tests-Data'!

!Morph methodsFor: '*BinaryModules' stamp: 'lw 1/15/2013 22:57'!
addStandardHaloMenuItemsTo: aMenu hand: aHandMorph
	"Add standard halo items to the menu"

	| unlockables |

	self isWorldMorph ifTrue:
		[^ self addWorldHaloMenuItemsTo: aMenu hand: aHandMorph].

	self mustBeBackmost ifFalse:
		[aMenu add: 'send to back' translated action: #goBehind.
		aMenu add: 'bring to front' translated action: #comeToFront.
		self addEmbeddingMenuItemsTo: aMenu hand: aHandMorph.
		aMenu addLine].

	self addFillStyleMenuItems: aMenu hand: aHandMorph.
	self addBorderStyleMenuItems: aMenu hand: aHandMorph.
	self addDropShadowMenuItems: aMenu hand: aHandMorph.
	self addLayoutMenuItems: aMenu hand: aHandMorph.
	self addHaloActionsTo: aMenu.
	owner isTextMorph ifTrue:[self addTextAnchorMenuItems: aMenu hand: aHandMorph].
	aMenu addLine.
	self addToggleItemsToHaloMenu: aMenu.
	aMenu addLine.
	self addCopyItemsTo: aMenu.
	self addPlayerItemsTo: aMenu.
	self addExportMenuItems: aMenu hand: aHandMorph.
	self addStackItemsTo: aMenu.
	self addMiscExtrasTo: aMenu.
	Preferences noviceMode ifFalse:
		[self addDebuggingItemsTo: aMenu hand: aHandMorph].

	aMenu addLine.
	aMenu defaultTarget: self.

	aMenu addLine.

	unlockables := self submorphs select:
		[:m | m isLocked].
	unlockables size = 1 ifTrue:
		[aMenu
			add: ('unlock "{1}"' translated format: unlockables first externalName)
			action: #unlockContents].
	unlockables size > 1 ifTrue:
		[aMenu add: 'unlock all contents' translated action: #unlockContents.
		aMenu add: 'unlock...' translated action: #unlockOneSubpart].
	
	BMPackage add: self toBinaryModuleSubmenuTo: aMenu.
	
	aMenu defaultTarget: aHandMorph.
! !

TestCase subclass: #BMPackageTest
	instanceVariableNames: 'class package'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules-Tests'!

!BMPackageTest methodsFor: 'utility' stamp: 'WS 1/14/2013 15:49'!
compareClass: aClass with: anotherClass

	self 
		assert: aClass name == anotherClass name;
		assert: aClass methodDict size equals: anotherClass methodDict size;
		assert: aClass instVarNames equals: anotherClass instVarNames;
	 	assert: aClass category equals: anotherClass category;
	 	assert: aClass superclass == anotherClass superclass.
		
	aClass methodDict associationsDo: [ :assoc | self assert: (anotherClass methodDict at: assoc key) size 
														equals: assoc value size].
	self assert: aClass class methodDict size equals: anotherClass class methodDict size.
	aClass class methodDict associationsDo: [ :assoc | self assert: (anotherClass class methodDict at: assoc key) size 
														equals: assoc value size]! !

!BMPackageTest methodsFor: 'utility' stamp: 'RL 1/16/2013 10:41'!
serializeAndMaterializeSimpleClassDescription
	self serializeAndMaterializeSimpleClassDescription: [ :aMaterializedClass :aCopy |
		self compareClass: aCopy with: aMaterializedClass.
	].! !

!BMPackageTest methodsFor: 'utility' stamp: 'WS 2/19/2013 16:14'!
serializeAndMaterializeSimpleClassDescription: testBlock
	" destroys the content of the instance variable class "
	| aCopy aDescription aMaterializedClass |
	[aCopy := class copy. "To avoid comparing the class to itself."
	aDescription := BMClassDescription for: class.
	(Smalltalk at: class name ifAbsent: nil) ifNotNil: [ :obsoleteClass | obsoleteClass removeFromSystem].
	aMaterializedClass := aDescription installClass.
	aDescription installMethods.
	
	testBlock value: aMaterializedClass value: aCopy.
	
	self deny: aCopy == aMaterializedClass.]
		ensure: [aMaterializedClass ifNotNil: [
					self tearDownClass: aMaterializedClass]]! !

!BMPackageTest methodsFor: 'test graph' stamp: 'RL 1/16/2013 11:44'!
setupComplexTestClass
	
	class := Object subclass: #BMTestClass
		instanceVariableNames: 'lorem'
		classVariableNames: ''
		poolDictionaries: ''
		category: 'BinaryModules-Tests-Data'.
		
	class compile: 'ipsum ^true'.
	class compile: 'dolor ^false'.
	class compile: 'create ^ 1->2'.
	class class compile: 'sit: amet amet isNil. ^Object new'.

	^ class! !

!BMPackageTest methodsFor: 'test graph' stamp: 'WS 1/14/2013 15:49'!
setupTestClass
	
	class := Object subclass: #BMTestClass
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'BinaryModules-Tests-Data'.
		
	class compile: 'foo ^true'.
	class compile: 'bar ^false'.

	^ class! !

!BMPackageTest methodsFor: 'test graph' stamp: 'lw 1/15/2013 22:52'!
simpleTestClass
	
	class := Object subclass: #BMSimpleTestClass
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'BinaryModules-Tests-Data'.
	
	^ class! !

!BMPackageTest methodsFor: 'running' stamp: 'WS 2/19/2013 16:56'!
tearDown

	super tearDown.
	class ifNotNil: [class removeFromSystem].
	package ifNotNil: [package removeFromSystem]! !

!BMPackageTest methodsFor: 'utility' stamp: 'WS 1/14/2013 15:49'!
tearDownClass: testClass
	"testClass superclass removeSubclass: testClass"
	
	testClass removeFromSystem! !

!BMPackageTest methodsFor: 'testing' stamp: 'RL 1/16/2013 11:59'!
testCallComplexMaterializedMethods
	class := self setupComplexTestClass.
	self serializeAndMaterializeSimpleClassDescription: [ :materializedClass :copy |
		| object |
		object := materializedClass new.
		self assert: [ object ipsum ].
		self deny: [ object dolor ].
		self assert: [ object create = (1->2) ].
		materializedClass sit: nil.
	].! !

!BMPackageTest methodsFor: 'testing' stamp: 'RL 1/16/2013 10:44'!
testCallSimpleMaterializedMethods
	class := self setupTestClass.
	self serializeAndMaterializeSimpleClassDescription: [ :materializedClass :copy |
		| object |
		object := materializedClass new.
		self assert: [ object foo ].
		self deny: [ object bar ].
	].! !

!BMPackageTest methodsFor: 'testing' stamp: 'WS 2/19/2013 16:17'!
testClassDescriptionCreation

	|aDescription|

	aDescription := BMClassDescription for: (Smalltalk at: #ExampleProject ifAbsent: [ self fail ]).
	self assert: aDescription name equals: #ExampleProject.
	self assert: [ aDescription represents: (Smalltalk at: #ExampleProject ifAbsent: [ self fail ]) ].
! !

!BMPackageTest methodsFor: 'testing' stamp: 'WS 2/19/2013 16:56'!
testCreatePackage
	
	| installation |
	self setupTestClass.
	installation := BMInstallation 
						named: 'testPackage' 
						withClasses: {class} 
						andObjects: {class new}.
	package := installation pack.
	self assert: installation classes first equals: class.
	self assert: installation objects first class equals: class! !

!BMPackageTest methodsFor: 'testing' stamp: 'WS 2/19/2013 16:57'!
testDeserializePackage
	
	| aClassCopy  aPackage |
	self setupTestClass.
	((BMInstallation
			named: 'testPackage' 
			withClasses: {class} 
			andObjects: {class new}) 
		exportToFile: 'testPackage') 
			removeFromSystem.
	aClassCopy := class copy.
	class removeFromSystem.
	package := BMPackage fromFile: 'testPackage'.
	self 
		should: [FLMaterializer materializeFromByteArray: package serializedObjects] 
		raise: FLClassNotFound.
	aPackage := package install.
	self compareClass: aClassCopy with: (Smalltalk at: aPackage classes first name).
	self assert: aPackage objects first class equals: (Smalltalk at: aPackage classes first name).! !

!BMPackageTest methodsFor: 'testing' stamp: 'WS 1/14/2013 15:48'!
testMaterializeClassDescription
	class := self setupTestClass.
	self serializeAndMaterializeSimpleClassDescription! !

!BMPackageTest methodsFor: 'testing' stamp: 'WS 1/14/2013 15:48'!
testMaterializeComplexClassDescription
	"self new testMaterializeComplexClassDescription"
	class := self setupComplexTestClass.
	self serializeAndMaterializeSimpleClassDescription! !

!BMPackageTest methodsFor: 'testing' stamp: 'lw 1/15/2013 22:51'!
testMaterializeSimpleClassDescription
	class := self simpleTestClass.
	self serializeAndMaterializeSimpleClassDescription! !

!BMPackageTest methodsFor: 'testing' stamp: 'WS 2/19/2013 16:57'!
testSerializePackage
	
	| aClass |
	aClass := self setupTestClass.
	package := (BMInstallation 
			named: 'testPackage' 
			withClasses: {aClass} 
			andObjects: {aClass new}) 
		exportToFile: 'testPackage'.
	self assert: (FileDirectory default fileExists: 'testPackage').
	FileDirectory default deleteFileNamed: 'testPackage'.
	self deny: (FileDirectory default fileExists: 'testPackage').
	aClass removeFromSystem! !

!BMPackageTest methodsFor: 'testing' stamp: 'WS 2/19/2013 16:57'!
testUninstallPackage
	
	self setupTestClass.
	package := (BMInstallation 
				named: 'testPackage' 
				withClasses: {class} 
				andObjects: {class new}) pack.
	self assert: package isInstalled.
	self assert: class allInstances size equals: 1.
	[package uninstall] on: Warning do: [ :warning | warning resume ].
	Smalltalk garbageCollect.
	self deny: package isInstalled.
	self assert: class isObsolete.
	self assert: class allInstances size equals: 0.! !

Object subclass: #BMBundle
	instanceVariableNames: 'classDescriptions serializedObjects name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMBundle commentStamp: 'lw 1/15/2013 23:20' prior: 0!
A BMBundle is a shortliving object. It is meant to be serialized to a file and give documented access to such a file, much like a struct.
It is our choice over a Dictionary with (somewhere) well-defined keys, in hope that is it more transparent ( in the not software library meaning ).
!

!BMBundle class methodsFor: 'instance creation' stamp: 'lw 1/13/2013 18:25'!
fromFile: aFileName
	| bundle |
	bundle := FLMaterializer materializeFromFileNamed: aFileName.
	^ (bundle isKindOf: self)
		ifTrue: [ bundle ]
		ifFalse: [ nil ]! !

!BMBundle class methodsFor: 'instance creation' stamp: 'lw 1/14/2013 00:50'!
named: aName classDescriptions: someBMClassDescriptions serializedObjects: someBinary
	^self new
		name: aName asString;
		classDescriptions: someBMClassDescriptions;
		serializedObjects: someBinary;
		yourself      ! !

!BMBundle methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:42'!
classDescriptions
	^ classDescriptions! !

!BMBundle methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:43'!
classDescriptions: aCollection
	classDescriptions := aCollection! !

!BMBundle methodsFor: 'accessing' stamp: 'WS 1/25/2013 16:31'!
linearizedClassDescriptions
	^ classDescriptions asSortedCollection: [ :c1 :c2 | (c1 inheritsFrom: c2) not ]! !

!BMBundle methodsFor: 'accessing' stamp: 'lw 1/15/2013 23:07'!
name
	^name! !

!BMBundle methodsFor: 'accessing' stamp: 'lw 1/15/2013 23:07'!
name: aName
	name := aName! !

!BMBundle methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:42'!
serializedObjects
	^ serializedObjects! !

!BMBundle methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:43'!
serializedObjects: aByteArray
	serializedObjects := aByteArray! !

Object subclass: #BMClassDescription
	instanceVariableNames: 'name instanceVariables classVariables category classMethods instanceMethods superclass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMClassDescription class methodsFor: 'instance creation' stamp: 'WS 2/19/2013 16:12'!
for: aClass
	^ self for: aClass classRegistry: IdentityDictionary new! !

!BMClassDescription class methodsFor: 'instance creation' stamp: 'WS 1/25/2013 16:18'!
for: aClass classRegistry: classRegistry
	^ super new
		initializeFrom: aClass classRegistry: classRegistry;
		yourself! !

!BMClassDescription class methodsFor: 'instance creation' stamp: 'WS 1/9/2013 18:08'!
new
	^ self shouldNotImplement! !

!BMClassDescription methodsFor: 'as yet unclassified' stamp: 'WS 1/25/2013 16:46'!
inheritsFrom: aClassOrClassDescription
	^ (superclass isKindOf: BMClassDescription) ifTrue: [
		superclass == aClassOrClassDescription or: [ superclass inheritsFrom: aClassOrClassDescription ]
	] ifFalse: [ "superclass isKindOf: String"
		false
	]! !

!BMClassDescription methodsFor: 'as yet unclassified' stamp: 'WS 1/25/2013 18:10'!
initializeFrom: aClass classRegistry: classRegistry
	name := aClass name.
	superclass := classRegistry at: aClass superclass ifAbsent: [ aClass superclass name ].
	instanceVariables := aClass instVarNames.
	classVariables := aClass classVarNames.
	category := aClass category.
	instanceMethods := aClass methodDict collect: [ :each | BMCompiledMethod for: each].
	classMethods := aClass class methodDict collect: [ :each | BMCompiledMethod for: each].! !

!BMClassDescription methodsFor: 'as yet unclassified' stamp: 'WS 1/25/2013 17:58'!
installClass
	(Smalltalk at: name asSymbol ifAbsent: nil) ifNotNil: [self error: 'Class already exists: ', name asString ].
	^ (Smalltalk at: ((superclass isKindOf: BMClassDescription) ifTrue: [ superclass name ] ifFalse: [ superclass ]) ) subclass: name
		instanceVariableNames: (String streamContents: [ :stream | instanceVariables do: [ :each | stream << each <<$ ] ])
		classVariableNames: (String streamContents: [ :stream | classVariables do: [ :each | stream << each <<$ ] ])
		poolDictionaries: ''
		category: category.! !

!BMClassDescription methodsFor: 'as yet unclassified' stamp: 'WS 1/25/2013 15:46'!
installMethods
	| newClass |
	newClass := Smalltalk at: name ifAbsent: [ self installClass ].
	
	instanceMethods do: [ :aMethod | aMethod addTo: newClass].
	classMethods do: [ :aMethod | aMethod addTo: newClass class].
	
	^ newClass! !

!BMClassDescription methodsFor: 'as yet unclassified' stamp: 'dg 1/10/2013 10:50'!
name
	^name! !

!BMClassDescription methodsFor: 'as yet unclassified' stamp: 'WS 1/23/2013 12:00'!
represents: aClass

	instanceVariables = aClass instVarNames ifFalse: [^false].
	category = aClass category ifFalse: [^false].
	instanceMethods size = aClass methodDict size ifFalse: [^false].
	
	^true! !

Object subclass: #BMCompiledMethod
	instanceVariableNames: 'bytecode protocol source selector headerWord literals'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMCompiledMethod class methodsFor: 'instance creation' stamp: 'dg 1/10/2013 11:11'!
for: aMethod

	^super new
			initializeFrom: aMethod; 
			yourself! !

!BMCompiledMethod class methodsFor: 'instance creation' stamp: 'dg 1/10/2013 11:09'!
new

	^self shouldNotImplement! !

!BMCompiledMethod methodsFor: 'as yet unclassified' stamp: 'RL 1/21/2013 17:55'!
addTo: aClass
	| method |
	method := self recreateFor: aClass.
	aClass addSelectorSilently: selector withMethod: method.
	aClass organization classify: method selector under: protocol.! !

!BMCompiledMethod methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 10:53'!
initializeFrom: aMethod

	protocol := aMethod methodClass organization categoryOfElement: aMethod selector.
	selector := aMethod selector.
	source := aMethod getSource.
	bytecode := ByteArray newFrom: aMethod.
	literals := aMethod literals collect: [ :literal | self packLiteral: literal ].
	headerWord := aMethod header! !

!BMCompiledMethod methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:06'!
packLiteral: literal
	^ BMLiteral from: literal! !

!BMCompiledMethod methodsFor: 'as yet unclassified' stamp: 'lw 1/10/2013 16:23'!
printOn: aStream

	super printOn: aStream.
	aStream << '(#' << selector << ', ' << headerWord << ', ' << bytecode size << ')'! !

!BMCompiledMethod methodsFor: 'as yet unclassified' stamp: 'RL 1/21/2013 17:48'!
recreateFor: aClass

	| aCompiledMethod |
	aCompiledMethod := CompiledMethod newFrom: bytecode.
	aCompiledMethod methodClass: aClass.
	literals withIndexDo: [ :literal :index |
			aCompiledMethod literalAt: index put: (self unpackLiteral: literal)
	].
	
	aCompiledMethod properties at: #source put: source.
	
	^ aCompiledMethod! !

!BMCompiledMethod methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:06'!
unpackLiteral: literal
	^ literal asLiteral! !

Object subclass: #BMInstallation
	instanceVariableNames: 'package classes objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMInstallation commentStamp: 'lw 1/15/2013 22:55' prior: 0!
A BMInstallation is the installed version of a BMPackage. It contains an Array of all Classes belonging to that Package.

Instance Variables
	classes:		<Object>
	name:		<Object>
	objects:		<Object>

classes
	- xxxxx

name
	- xxxxx

objects
	- xxxxx
!

!BMInstallation class methodsFor: 'instance creation' stamp: 'WS 2/19/2013 16:51'!
installFromFileNamed: aFileName
	^ (BMPackage fromFile: aFileName)
		ifNil: [ self error: 'Error during package load' ]
		ifNotNil: [ :package | package install ]! !

!BMInstallation class methodsFor: 'instance creation' stamp: 'lw 1/14/2013 01:10'!
named: name
	^ super new
		name: name asString;
		classes: #();
		objects: #();
		yourself! !

!BMInstallation class methodsFor: 'instance creation' stamp: 'lw 1/14/2013 00:50'!
named: aString withClasses: classes andObjects: objects
	^ (self named: aString asString)
		classes: classes;
		objects: objects;
		yourself! !

!BMInstallation class methodsFor: 'instance creation' stamp: 'RL 1/9/2013 11:18'!
new
	self shouldNotImplement! !

!BMInstallation class methodsFor: 'instance creation' stamp: 'lw 1/15/2013 23:15'!
of: aPackage with: someClasses and: someObjects
	^super new
		of: aPackage;
		classes: someClasses;
		objects: someObjects;
		yourself! !

!BMInstallation class methodsFor: 'as yet unclassified' stamp: 'WS 1/9/2013 18:19'!
removeFromSystem
	^ self notYetImplemented! !

!BMInstallation methodsFor: 'accessing - derived' stamp: 'lw 1/15/2013 23:04'!
addClass: aClass
	
	self assert: aClass isMeta not.
	(classes includes: aClass) ifFalse: 
		[classes := classes copyWith: aClass].
	self changed: #addClass: with: aClass.
	^aClass! !

!BMInstallation methodsFor: 'system integration' stamp: 'WS 1/14/2013 11:26'!
addMorph: aMorph
	
	self inform: 'Remember to also add the classes neccessary to create the morph.'.
	self addObject: aMorph  ! !

!BMInstallation methodsFor: 'accessing - derived' stamp: 'lw 1/15/2013 23:04'!
addObject: anObject
	
	(objects includes: anObject) ifFalse:
		[objects := objects copyWith: anObject].
	self changed: #addObject: with: anObject.
	^anObject! !

!BMInstallation methodsFor: 'accessing' stamp: 'WS 1/9/2013 18:02'!
classes
	^ classes! !

!BMInstallation methodsFor: 'accessing' stamp: 'lw 1/13/2013 18:18'!
classes: aCollection
	classes := aCollection.
	self changed: #classes: with: aCollection! !

!BMInstallation methodsFor: 'package behavior' stamp: 'lw 1/15/2013 23:20'!
exportToFile
	^ self pack exportToFile! !

!BMInstallation methodsFor: 'package behavior' stamp: 'lw 1/15/2013 23:21'!
exportToFile: aFileName
	^ self pack exportToFile: aFileName! !

!BMInstallation methodsFor: 'package behavior' stamp: 'WS 2/19/2013 16:58'!
extractToFile: aFileName
	^ self pack
		extractToFile: aFileName;
		yourself! !

!BMInstallation methodsFor: 'package behavior' stamp: 'lw 1/15/2013 23:21'!
name: aName
	^self pack name: aName! !

!BMInstallation methodsFor: 'accessing' stamp: 'WS 1/9/2013 18:03'!
objects
	^ objects! !

!BMInstallation methodsFor: 'accessing' stamp: 'lw 1/13/2013 18:19'!
objects: aCollection
	objects := aCollection.
	self changed: #objects: with: aCollection! !

!BMInstallation methodsFor: 'accessing - derived' stamp: 'lw 1/15/2013 23:16'!
of: aPackage
	self assert: package isNil.
	package := aPackage! !

!BMInstallation methodsFor: 'accessing - derived' stamp: 'lw 1/15/2013 23:20'!
pack
	package ifNil: [package := BMPackage fromInstallation: self].
	^package! !

Object subclass: #BMLiteral
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

BMLiteral subclass: #BMAssociationLiteral
	instanceVariableNames: 'key value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMAssociationLiteral class methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:10'!
from: anAssociation
	^ self new
		key: (BMLiteral from: anAssociation key);
		value: (BMLiteral from: anAssociation value);
		yourself! !

!BMAssociationLiteral methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:21'!
asLiteral
	^ key asLiteral -> value asLiteral! !

!BMAssociationLiteral methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:09'!
key
	^ key! !

!BMAssociationLiteral methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:09'!
key: aLiteral
	key := aLiteral! !

!BMAssociationLiteral methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:09'!
value
	^ value! !

!BMAssociationLiteral methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:09'!
value: aLiteral
	value := aLiteral! !

BMLiteral subclass: #BMClassLiteral
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMClassLiteral class methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:08'!
from: aClass
	^ self new
		name: aClass name;
		yourself! !

!BMClassLiteral methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:58'!
asLiteral
	^ Smalltalk at: name asSymbol ifAbsent: [ nil ]! !

!BMClassLiteral methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:07'!
name
	^ name! !

!BMClassLiteral methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:47'!
name: aName
	name := aName asString! !

BMClassLiteral subclass: #BMMetaClassLiteral
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMMetaClassLiteral class methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:35'!
from: aMetaclass
	^ self new
		name: aMetaclass theNonMetaClass name;
		yourself! !

!BMMetaClassLiteral methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:58'!
asLiteral
	^ super asLiteral class! !

BMLiteral subclass: #BMGlobalLiteral
	instanceVariableNames: 'global'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMGlobalLiteral class methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:16'!
from: aGlobal
	^ self new
		global: aGlobal;
		yourself! !

!BMGlobalLiteral methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:16'!
asLiteral
	^ global! !

!BMGlobalLiteral methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:15'!
global
	^ global! !

!BMGlobalLiteral methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:16'!
global: aGlobal
	global := aGlobal! !

!BMLiteral class methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 12:04'!
from: aLiteral
	(aLiteral isKindOf: Association)
		ifTrue: [ ^ BMAssociationLiteral from: aLiteral ].
	(aLiteral isKindOf: Class)
		ifTrue: [ ^ BMClassLiteral from: aLiteral ].
	(aLiteral isKindOf: Metaclass)
		ifTrue: [ ^ BMMetaClassLiteral from: aLiteral ].
	((aLiteral isKindOf: Symbol) | (aLiteral isKindOf: SmallInteger) | aLiteral isNil)
		ifFalse: [ Transcript show: 'create literal from unexpected class: '; show: aLiteral; show: ' '; show: aLiteral class asString; cr ].
	^ BMGlobalLiteral from: aLiteral! !

!BMLiteral methodsFor: 'as yet unclassified' stamp: 'RL 1/16/2013 11:04'!
asLiteral
	self subclassResponsibility! !

Object subclass: #BMPackage
	instanceVariableNames: 'installation bundle name'
	classVariableNames: 'BMPackages'
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMPackage commentStamp: 'WS 2/19/2013 16:42' prior: 0!
A BMPackage is a Stub enabling to find and install BMInstallations.

Instance Variables
	file:		<Object>
	name:		<Object>
	package:		<Object>

file
	Either an object which understands name, classeDescriptions and serializedObjects, or a String, which points to a file where we can find these information.
	We use the file to see that this package has been saved, and is not changed since then.
	Saving the filename instead of the struct, avoids saving the same information in two places in the image. In case of class-descriptions, that might not be much overhead, but they include CompiledMethod-Descriptions, with source, and serializedObjects size is unknown.

name
	A String used to find me. Should be the same as used in BMPackage and is copied to BMFile for serialization.

installation
	If the package is installed, this variable points to the BMINstallation instance, which contains the materialized classes and objects.
	If this variable is nil, the assumptions is, that it is not installed.!

!BMPackage class methodsFor: 'system integration' stamp: 'WS 2/19/2013 16:45'!
add: aMorph toBinaryModuleSubmenuTo: aMenu
	"add the possibility to add a morph to the object list of a BMInstallation"
	
	| subMenu |
	subMenu := MenuMorph new defaultTarget: self.
	
	subMenu add: 'new module...' translated selector: #createNewInstallationWith: argument: aMorph.
	BMPackages select: [ :each | each isInstalled ] thenDo: [ :each |
		subMenu 
			add: each menuString 
			target: each installation
			selector: #addMorph: 
			argument: aMorph].
	
	aMenu 
		addLine;
		add: 'add to Binary Module...' translated subMenu: subMenu;
		addLine! !

!BMPackage class methodsFor: 'system integration' stamp: 'lw 1/15/2013 23:21'!
createNewPackageWith: aMorph
	
	"Using #addMorph: instead of including the morph in the objects array, to enable adding classes to the classes array which are needed for the morph and its submorphs at only one point in BMInstallation."
	^(UIManager default request: 'Please type new module name' translated) 
		ifNotNil: [ :aName | 
			aName ifEmpty: [ nil ] ifNotEmpty: [
			((BMInstallation named: aName)
				addMorph: aMorph;
				pack) ] ]! !

!BMPackage class methodsFor: 'magic constants' stamp: 'WS 2/19/2013 16:46'!
fileExtension
	^ '.bundle'! !

!BMPackage class methodsFor: 'instance creation' stamp: 'WS 2/19/2013 16:49'!
fromFile
	"finds only files without extension and files ending with .bundle"
	^ (UIManager default chooseFileMatching: (Array with: '' with: self class fileExtension allButFirst))
		ifNotNil: [ :file | self fromFile: file ]! !

!BMPackage class methodsFor: 'instance creation' stamp: 'WS 2/19/2013 16:50'!
fromFile: aFileName
	
	^BMPackages 
		detect: [ :aPackage | [aPackage bundle = aFileName] on: Error do: [false]]
		ifNone: [ super new
					fromBinary: (BMBundle fromFile: aFileName);
					addToSystem;
					yourself ]! !

!BMPackage class methodsFor: 'instance creation' stamp: 'lw 1/15/2013 23:11'!
fromInstallation: anInstallation
	^ super new
		bundle: anInstallation;
		addToSystem;
		yourself! !

!BMPackage class methodsFor: 'class initialization' stamp: 'WS 2/19/2013 16:50'!
initialize
	super initialize.
	BMPackages := OrderedCollection new! !

!BMPackage class methodsFor: 'instance creation' stamp: 'WS 2/19/2013 16:46'!
loadNamed: aPackageName
	^ self fromFile: aPackageName, self fileExtension! !

!BMPackage class methodsFor: 'instance creation' stamp: 'WS 2/19/2013 16:50'!
named: aString
	
	^BMPackages 
		detect: [ :ea | ea name = aString asString ]
		ifNone: [ (UIManager default confirm: 'The package was not found. Do you want to create it?' translated) ifFalse: [^nil].
			(BMInstallation named: aString asString) pack
				addToSystem;
				yourself]! !

!BMPackage class methodsFor: 'instance creation' stamp: 'WS 1/9/2013 18:01'!
new
	self shouldNotImplement! !

!BMPackage methodsFor: 'installation facade' stamp: 'lw 1/15/2013 22:58'!
addClass: aClass
	^self installation addClass: aClass! !

!BMPackage methodsFor: 'installation facade' stamp: 'lw 1/17/2013 11:09'!
addClasses: someClasses
	someClasses do: [:aClass | self installation addClass: aClass].
	^someClasses! !

!BMPackage methodsFor: 'installation facade' stamp: 'lw 1/15/2013 22:58'!
addObject: anObject
	^self installation addObject: anObject    ! !

!BMPackage methodsFor: 'utilities' stamp: 'WS 2/19/2013 16:52'!
addToSystem
	BMPackages ifNil: [ BMPackages := OrderedCollection new].
	(BMPackages includes: self) 
		ifFalse: [BMPackages addLast: self]! !

!BMPackage methodsFor: 'accessing-derived' stamp: 'WS 1/25/2013 18:01'!
bundle
	"do we need to save the created file?"
	^bundle
		ifNil: [bundle := BMBundle 
				named: name 
				classDescriptions: (self createClassDescriptionsFor: self installation classes) 
				serializedObjects: (FLSerializer serializeToByteArray: self installation objects) ]
		ifNotNil: [bundle isSymbol | bundle isString 
					ifTrue: [bundle := BMBundle fromFile: bundle]
					ifFalse: [bundle]]! !

!BMPackage methodsFor: 'creation' stamp: 'lw 1/15/2013 23:12'!
bundle: anInstallation
	
	installation := anInstallation.
	installation addDependent: self! !

!BMPackage methodsFor: 'utilities' stamp: 'WS 1/25/2013 16:22'!
createClassDescriptionsFor: aCollection
	| classToDescriptions |
	classToDescriptions := IdentityDictionary new.
	(aCollection asSortedCollection: [ :c1 :c2 | (c1 inheritsFrom: c2) not ]) do: [ :class |
		classToDescriptions at: class put: (BMClassDescription for: class classRegistry: classToDescriptions)
	].
	^ classToDescriptions values! !

!BMPackage methodsFor: 'installation' stamp: 'WS 2/19/2013 16:19'!
createClasses
	| classDescriptions classes |
	self assert: bundle notNil.
	classDescriptions := self bundle linearizedClassDescriptions.
	classes := OrderedCollection new.
	
	classDescriptions do: [ :classDescription | classes add: classDescription installClass ].
	classDescriptions do: [ :classDescription | classDescription installMethods ].
	
	^ classes! !

!BMPackage methodsFor: 'saving' stamp: 'WS 2/19/2013 16:46'!
exportToFile
	^ self exportToFile: name, self class fileExtension! !

!BMPackage methodsFor: 'saving' stamp: 'lw 1/15/2013 23:23'!
exportToFile: aFileName
	FLSerializer serialize: self bundle toFileNamed: aFileName.
	bundle := aFileName! !

!BMPackage methodsFor: 'saving' stamp: 'lw 1/14/2013 01:15'!
extractToFile: aFileName
	self exportToFile: aFileName.
	self uninstall! !

!BMPackage methodsFor: 'creation' stamp: 'lw 1/15/2013 23:23'!
fromBinary: aBMFile
	name := aBMFile name.
	bundle := aBMFile! !

!BMPackage methodsFor: 'installation' stamp: 'lw 1/15/2013 23:24'!
install
	self assert: bundle notNil.
	self assert: installation isNil.
	
	installation := BMInstallation
		of: self
		with: self createClasses
		and: self objects.
	installation addDependent: self.
	^installation! !

!BMPackage methodsFor: 'accessing-derived' stamp: 'lw 1/15/2013 22:58'!
installation
	^installation ifNil: [self notify: 'This package is not installed.' ]! !

!BMPackage methodsFor: 'testing' stamp: 'lw 1/15/2013 22:59'!
isInstalled
	^installation notNil! !

!BMPackage methodsFor: 'accessing' stamp: 'WS 1/14/2013 11:02'!
menuString
	^ name! !

!BMPackage methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:40'!
name
	^ name! !

!BMPackage methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:40'!
name: aString
	name := aString! !

!BMPackage methodsFor: 'installation' stamp: 'WS 1/25/2013 18:04'!
objects
	self assert: bundle notNil.
	^ FLMaterializer materializeFromByteArray: self bundle serializedObjects! !

!BMPackage methodsFor: 'utilities' stamp: 'lw 1/17/2013 11:23'!
printOn: aStream
	aStream << name << '(' << (self isInstalled ifTrue: [''] ifFalse: 'not')<<' installed)'! !

!BMPackage methodsFor: 'utilities' stamp: 'WS 2/19/2013 16:54'!
removeFromSystem
	BMPackages ifNotNil: 
		[BMPackages := BMPackages reject: [ :each | each == self]]! !

!BMPackage methodsFor: 'accessing-derived' stamp: 'WS 2/19/2013 16:55'!
serializedObjects
	^bundle 
		ifNil: [self error: 'Package not created from file.']
		ifNotNil: [ bundle serializedObjects ]! !

!BMPackage methodsFor: 'installation' stamp: 'lw 1/15/2013 23:24'!
uninstall
	"although unload depends heavily on data of Installation, for symmetrie to install, it is implemented here"
	self isInstalled 
		ifTrue: [ 
			bundle ifNil: [self notify: 'Are you sure? The classes and objects will be irrevocably removed.'].
			self installation classes do: [ :aClass | aClass removeFromSystem].
			"Only for morphs, we know how to remove them."
			self installation objects do: [ :each | each isMorph ifTrue: [each delete]].
			installation := nil]! !

Object subclass: #BMSnippets
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules-Tests'!

!BMSnippets class methodsFor: 'as yet unclassified' stamp: 'WS 2/19/2013 16:43'!
profile
	Transcript 
		<< 'Profiling BM' << String cr.
	Transcript
		<< (self profileBM / 1000) asFloat << 's' << String cr.
	Transcript
		<< 'Profiling Moticello' << String cr.
	Transcript
		<< (self profileMonticello / 1000) asFloat << 's' << String cr.
	Transcript flush! !

!BMSnippets class methodsFor: 'as yet unclassified' stamp: 'WS 2/19/2013 16:43'!
profileBM
	^self profileSwaBallInstaller: [(BMPackage fromFile: 'swaexample.pkg') install]  ! !

!BMSnippets class methodsFor: 'as yet unclassified' stamp: 'WS 2/19/2013 16:44'!
profileMonticello
	^self profileSwaBallInstaller: [MczInstaller installFileNamed: 'SwaExample-tfel.1.mcz']  ! !

!BMSnippets class methodsFor: 'as yet unclassified' stamp: 'WS 2/19/2013 16:44'!
profileSwaBallInstaller: installationBlock
	
	| sum |
	sum := 0.
	1 to: 10 do: [ :i | 
		sum := sum + installationBlock timeToRunWithoutGC.
		(PackageInfo named: #SwaExample) classes do: [:ea | ea removeFromSystem].
		Smalltalk garbageCollect].
	^sum! !

Object subclass: #ExampleProject
	instanceVariableNames: 'foo bar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules-Tests-Data'!

ExampleProject subclass: #ExampleProjectSubclass
	instanceVariableNames: 'foo2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules-Tests-Data'!

!ExampleProjectSubclass methodsFor: 'as yet unclassified' stamp: 'WS 1/25/2013 18:05'!
barfoo
	^ self! !

!MultiByteFileStream methodsFor: '*BinaryModules' stamp: 'lw 12/21/2012 14:52'!
nextPutAll: aCollection

	(self isBinary or: [aCollection class == ByteArray]) ifTrue: [
		^ super nextPutAll: aCollection.
	].
	aCollection class == WordArrayForSegment ifTrue: [
		^ super nextPutAll: (Bitmap withAll: aCollection) asByteArray
	].
	^converter nextPutAll: aCollection toStream: self! !
BMPackage initialize!
